/*
 * Copyright (c) 2010, 2011, 2014, 2016 Paul de Weerd <weerd@weirdnet.nl>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/param.h>

#include <err.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define MAX_BOUND 1LL<<32

#ifndef __dead
#define __dead
#endif

__dead void usage(void);

int
main(int argc, char *argv[]) {
	long long bnd = -1, cnt = 1;
	u_int32_t b32 = 0;
	int quiet = 0, userbound = 0, c;
	const char *errstr;
	char *envbound;

	if (pledge("stdio", NULL) == -1)
		err(1, "pledge");

	envbound = getenv("RANVAL_BOUND");
	if (envbound != NULL) {
		bnd = strtonum(envbound, 1, MAX_BOUND, &errstr);
		if (errstr) {
			bnd = -1;
		} else {
			userbound = 1;
		}
	}
		
	while ((c = getopt(argc, argv, "b:c:q")) != -1)
		switch (c) {
		case 'b':
			bnd = strtonum(optarg, 1, MAX_BOUND, &errstr);
			if (errstr)
				errx(1, "bound is %s: %s", errstr, optarg);
			userbound = 1;
			break;
		case 'c':
			cnt = strtonum(optarg, 0, LLONG_MAX, &errstr);
			if (errstr)
				errx(1, "count is %s: %s", errstr, optarg);
			break;
		case 'q':
			quiet = 1;
			break;
		default:
			usage();
		}

	argc -= optind;
	argv += optind;

	if (cnt != 1 && quiet == 1)
		errx(1, "quiet mode needs -c 1");

	if (argc > 1)
		usage();

	if (argc == 1) {
		if (userbound == 1) {
			usage();
		} else {
			bnd = strtonum(*argv, 1, MAX_BOUND, &errstr);
			if (errstr)
				errx(1, "bound is %s: %s", errstr, *argv);
			userbound = 1;
		}
	}

	if (bnd == MAX_BOUND) {
		userbound = 0;
	} else {
		b32 = (u_int32_t)bnd;
	}

	if (quiet == 1) {
		if (b32 > 256)
			b32 = 256;
		return ((int)arc4random_uniform(b32));
	} else {
		if (cnt == 0) {
			for (;;)
				if (userbound == 1) {
					printf("%u\n", arc4random_uniform(b32));
				} else {
					printf("%u\n", arc4random());
				}
		} else {
			for(; cnt != 0; cnt--)
				if (userbound == 1) {
					printf("%u\n", arc4random_uniform(b32));
				} else {
					printf("%u\n", arc4random());
				}
			return (0);
		}
	}
}

__dead void
usage(void)
{
	extern char *__progname;

	printf("usage: %s [-q | -c count] [[-b] bound]\n", __progname);
	exit(1);
}
